class OrderValidationException(Exception):
    pass


class BenefitExpiredOrInactiveException(OrderValidationException):
    pass


class PodTypeNotValidException(OrderValidationException):
    pass


class InsufficientSaleAmountException(OrderValidationException):
    pass


class ErrorApplyingCoupon(OrderValidationException):
    pass


class ExcessiveSaleAmountException(OrderValidationException):
    pass


class UserLevelNotAllowedException(OrderValidationException):
    pass


class MissingItemsException(OrderValidationException):
    pass


class CustomBenefitAlreadyAppliedException(OrderValidationException):
    pass


class InvalidServiceTypeException(OrderValidationException):
    pass


class InvalidSaleTypeException(OrderValidationException):
    pass


class InvalidMinSeatsException(OrderValidationException):
    pass


class InvalidSectorException(OrderValidationException):
    pass


class ApplyingDiscountUnknownException(OrderValidationException):
    pass


class OrderPictureException(OrderValidationException):
    pass


class InvalidAddItemsInTotaledOrderException(OrderValidationException):
    pass


class InvalidServicePeriodOrderException(OrderValidationException):
    pass


class InvalidBrandException(OrderValidationException):
    pass


class AddItemsException(OrderValidationException):
    pass
