from typing import List, Dict


class BenefitList(object):
    def __init__(
            self,
            benefit_pre_total_sequence,         # type: List
            manual_benefits_on_total_sequence,  # type: List
            discount_item_values,               # type: Dict
            discount_id_tip_excluded,           # type: List
            tender_id_tip_excluded,             # type: List
            benefits                            # type: List
    ):
        # type: (...) -> None

        self.benefit_pre_total_sequence = benefit_pre_total_sequence
        self.manual_benefits_on_total_sequence = manual_benefits_on_total_sequence
        self.discount_item_values = discount_item_values
        self.discount_id_tip_excluded = discount_id_tip_excluded
        self.tender_id_tip_excluded = tender_id_tip_excluded
        self.benefit_list = benefits
