import os
import json


class GetBannersResponse(object):
    def __init__(
        self,
        totem_config_path  # type: str
    ):
        # type: (...) -> None

        if not totem_config_path or not os.path.exists(totem_config_path):
            return

        config_file = open(totem_config_path, 'rb')
        data = json.load(config_file)
        self.banners = data.get("banner", {})
