from datetime import datetime


class OpenedTable(object):
    def __init__(
        self,
        table_id,       # type: str
        name,           # type: str
        sector,         # type: str
        opened_date,    # type: datetime
        status,         # type: str
        operator,       # type: str
        seats,          # type: seats
    ):
        # type: (...) -> None

        self.table_id = table_id
        self.name = name
        self.sector = sector
        self.opened_date = opened_date
        self.status = status
        self.operator = operator
        self.amount = 0
        self.seats = seats
        self.orders = []
