class InvalidResponseMessage(Exception):
    def __init__(self, reason):
        self.reason = reason

    def __str__(self):
        return self.__repr__()

    def __repr__(self):
        return "InvalidResponseMessage(reason='')".format(self.reason)
