from services.services import UpdateService


class UpdateAlertsInteractor(object):
    def __init__(
        self,
        update_service,         # type: UpdateService
    ):
        # type: (...) -> None

        self.update_service = update_service

    def process_request(
        self,
    ):
        # type: (...) -> None

        self.update_service.update_configured_alerts()
