from typing import Optional


class TableReportSaleLineDto(object):

    def __init__(self):
        self.product_name = ""              # type: str
        self.price = 0.0                    # type: float
        self.qty = 0                        # type: float
        self.default_qty = 0                # type: Optional[int]
        self.item_type = ""                 # type: str
        self.seat = None                    # type: Optional[int]
        self.level = 0                      # type: int
        self.comment = ""                   # type: Optional[str]
        self.line_number = 0                # type: int
        self.service_period = None          # type: Optional[str]
        self.order_id = None                # type: Optional[int]
        self.item_id = None                 # type: Optional[str]
        self.part_code = None               # type: Optional[int]
        self.overwritten_unit_price = None  # type: Optional[float]
        self.fraction_parent = None         # type: Optional[int]
