class CleanDatabasesRequest(object):
    def __init__(
        self,
        default_outdated_events_days,  # type: int
    ):
        # type: (...) -> None

        self.default_outdated_events_days = default_outdated_events_days
