class TenderType(object):
    MONEY = 0
    CREDIT_CARD = 1
    DEBIT_CARD = 2


class OrderTender(object):
    def __init__(self, type, value):
        # type: (int, float) -> None
        self.type = type
        self.value = value
